///////////////////////////////////////////////////////////////////////////////
//
// Time-stamp: "06.11.10 21:02 AVR_LCD44780.cpp klaus?wachtler.de"
// $Id: AVR_LCD44780.cpp 529 2010-11-07 23:55:25Z klaus $
//
////////////////////////////////////////////////////////////////////////////////
//
// Beispiel für Anschluß Text-LCD mit Controller 44780
//
/////////////////////////////////////////////////////////////////////////////
//
//  Beispielanschluss LCD 4x40 an ATMega16/32:
//
//                                  +----_----+
//  LCD-11-DB4 ------  (XCK/T0) PB0 |  1   40 | PA0 (ADC0)
//  LCD-12-DB5 ------      (T1) PB1 |  2   39 | PA1 (ADC1)
//  LCD-13-DB6 ---- (INT2/AIN0) PB2 |  3   38 | PA2 (ADC2)
//  LCD-14-DB7 ----- (OC0/AIN1) PB3 |  4   37 | PA3 (ADC3)
//  LCD- 4-RS ----------- (-SS) PB4 |  5   36 | PA4 (ADC4)
//  LCD- 6-EN1 --------- (MOSI) PB5 |  6   35 | PA5 (ADC5)
//  LCD-15-EN2 --------- (MISO) PB6 |  7   34 | PA6 (ADC6)
//                        (SCK) PB7 |  8   33 | PA7 (ADC7)
//                           -RESET |  9   32 | AREF
//                              VCC | 10   31 | GND
//                              GND | 11   30 | AVCC
//                            XTAL2 | 12   29 | PC7 (TOSC2)
//                            XTAL1 | 13   28 | PC6)(TOSC1)
//                        (RxD) PD0 | 14   27 | PC5)(TDI)
//                        (TxD) PD1 | 15   26 | PC4 (TDO)
//                       (INT0) PD2 | 16   25 | PC3 (TMS)
//                       (INT1) PD3 | 17   24 | PC2 (TCK)
//                       (OC1B) PD4 | 18   23 | PC1 (SDA)
//                       (OC1A) PD5 | 19   22 | PC0 (SCL)
//                       (ICP1) PD6 | 20   21 | PD7 (OC2)
//                                  +---------+
//
// (LCD-EN2 gibt es nur bei 4x40.)
//
// Sowie die restlichen LCD-Leitungen:
//   VSS  1-----GND
//   VDD  2-----+5V
//   VEE  3--Abgriff Poti 10k zw. GND und +5V
//   R/W  5-----GND
//   DB0  7-NC
//   DB1  8-NC
//   DB2  9-NC
//   DB3 10-NC
// LED-A (17 bei 4x40, sonst 15)--R 220 Ohm--+5V;  Achtung! R je nach LCD!
// LED-K (18 bei 4x40, sonst 16)-----GND
//
////////////////////////////////////////////////////////////////////////////////



#include <stdio.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <inttypes.h>
#include <util/delay.h>

#include "LCD44780.h"


/////////////////////////////////////////////////////////////////////////////


// getestet mit Displaytech 404b
//void demo_4x40()
//{
//  // Initialisierung mit Angabe des Typs und Pinbelegung der
//  // Anschluesse.
//  //
//  // Moegliche Typen:
//  //    LCD_Type_1x8
//  //    LCD_Type_1x16_MUX8
//  //    LCD_Type_1x16_MUX16
//  //    LCD_Type_1x20
//  //    LCD_Type_1x40
//  //    LCD_Type_2x8
//  //    LCD_Type_2x12
//  //    LCD_Type_2x16
//  //    LCD_Type_2x20
//  //    LCD_Type_2x24
//  //    LCD_Type_2x40
//  //    LCD_Type_4x16
//  //    LCD_Type_4x20
//  //    LCD_Type_4x40
//  //
//  // Mit 1x16 gibt es zwei Typen:
//  // LCD_Type_1x16_MUX8 kann alle 16 Zeichen direkt nacheinander
//  // ausgeben.
//  // Bei LCD_Type_1x16_MUX16 dagegen liegen nur die ersten 8 Zeichen
//  // im Speicher direkt hintereinander, irgendwo anders kommen dann
//  // die letzten 8 Zeichen. Deshalb kann man damit in der aktuellen
//  // Version vermutlich nicht direkt über die Mitte
//  // rueberschreiben. Vielmehr muss man an den Anfang positionieren,
//  // kann 8 Zeichen ausgeben, muss dann in die Spalte 8 positionieren
//  // und kann den Rest ausgeben.
//  // Wenn ich so ein Ding mal bekomme, aendere ich das Verhalten
//  // vielleicht so, dass es sinnvoller wird.
//  //
//  // Die Port- und Pinbelegung muss die 4 oberen Datenleitungen, RS
//  // und Enable bzw. Enable1 definieren (die DDR werden intern aus den
//  // PORTs abgeleitet).
//  // Bei LCD_Type_4x40 sind zusaetzlich Port und Pin fuer Enable2
//  // anzugeben.
//  //
//
//  AnyWare::LCD44780    lcd1( AnyWare::LCD44780::LCD_Type_4x40,
//                             PORTD, 0, // Port, Pin LCD-DB4
//                             PORTD, 1, // Port, Pin LCD-DB5
//                             PORTD, 2, // Port, Pin LCD-DB6
//                             PORTD, 3, // Port, Pin LCD-DB7
//                             PORTD, 4, // Port, Pin LCD-RS
//                             PORTD, 5, // Port, Pin LCD-Enable1
//                             PORTD, 7  // Port, Pin LCD-Enable2 (nur bei 4x40)
//                             );
//
//  // Es koennen mehrere LCDs gleichzeitig betrieben werden.
//  // Dabei duerfen die Leitungen parallel angeschlossen sein; nur die
//  // Enable-Signale muessen sich unterscheiden.
//  //AnyWare::LCD44780    lcd2( AnyWare::LCD44780::LCD_Type_2x40,
//  //                           PORTD, 0, // Port, Pin LCD-DB4
//  //                           PORTD, 1, // Port, Pin LCD-DB5
//  //                           PORTD, 2, // Port, Pin LCD-DB6
//  //                           PORTD, 3, // Port, Pin LCD-DB7
//  //                           PORTD, 4, // Port, Pin LCD-RS
//  //                           PORTD, 6, // Port, Pin LCD-Enable1
//  //                           );
//
//
//
//  // Nach dem Initialisieren steht die Ausgabeposition an Stelle x=0 (Spalte,
//  // ganz links) und y=0 (Zeile, ganz oben) und der Bildschirm ist gelöscht.
//
//  // printChar() gibt ein Zeichen an der aktuellen Ausgabeposition aus
//  // und erhoeht diese um eins nach rechts, soweit noch Platz in der
//  // Zeile ist.
//  // Bei Ueberschreiten des rechten Rands unterscheidet sich das
//  // Verhalten je nach LCD-Typ.
//
//  lcd1.printChar( 'H' );
//  lcd1.printChar( 'a' );
//  lcd1.printChar( 'l' );
//  lcd1.printChar( 'l' );
//  lcd1.printChar( 'o' );
//  lcd1.printChar( '!' );
//  lcd1.printChar( ' ' );
//
//  // Mit printf() kann fast wie gewohnt gearbeitet werden.
//  // Die Ausgabe laeuft ueber einen internen Puffer von 40 Zeichen;
//  // wenn diese Laenge nicht reicht, wird abgebrochen und nichts
//  // ausgegeben.
//  // Zeilenvorschub mit \n ist nicht moeglich.
//  lcd1.printf( "i ist %4d, was sonst?", -1230 );
//
//  // Ausgabeposition setzen:
//  lcd1.setCursor( 0, 1 );
//  // Kleine Umlaute sind vorhanden, aber an anderen Stellen (siehe [3]):
//  // ae = \xe1, oe = \xef, ue = \xf5, sz = \xe2
//  // Grosse Umlaute gibt es nicht und koennten ueber eigene Zeichen
//  // selbst definiert werden (siehe unten).
//  lcd1.printf( "Nat\xf5rlich wei\xe2 %s wieder alles besser", "Guido" );
//
//  // Eine Variante von printf() bekommt drei weitere Parameter vorweg:
//  // - x-Position (gezaehlt ab 0)
//  // - y-Position (gezaehlt ab 0, von oben nach unten)
//  // - Ausgabebreite; soviele Zeichen werden ausgegeben (ggf. wird mit
//  //   Leerzeichen aufgefuellt; reicht die angegebene Laenge nicht für
//  //   die Ausgabe, dann werden '#' ausgegeben)
//  //
//  lcd1.printf( 0, 2, 40, "\xf5""berhaupt es geht voran: \xff\xff\xff\xff\xff\xff\xff" );
//
//  // Analog zu beiden printf()-Varianten gibt es noch die Variante
//  // printf_P(), bei der der auszugebende Formatstring aus dem
//  // Programmspeicher geholt wird.
//
//
//  // Zeichen (5x8 Pixel) neu definieren:
//  // Es werden 8 uint8_t definiert, in denen bitweise die Daten
//  // jeweils einer Pixelzeile liegen.
//  // In jeder uint8_t werden nur die unteren 5 Bit genutzt.
//  //
//  // Auf diese Weise kann man 8 Zeichen (0 bis 7) definieren, die sich
//  // an den Positionen 8-15 widerspiegeln.
//
//  // Zeichen 0 (oder 8, ist das selbe) soll ein grosses Ae (Umlaut) werden:
//  lcd1.defChar( 0,
//                (uint8_t[]){ 0b10001,
//                             0b01110,
//                             0b10001,
//                             0b10001,
//                             0b11111,
//                             0b10001,
//                             0b10001,
//                             0b00000,
//                           }
//                );
//
//  // Zeichen 1 (oder 9) soll ein grosses Oe (Umlaut) werden:
//  lcd1.defChar( 1,
//                (uint8_t[]){ 0b10001,
//                             0b01110,
//                             0b10001,
//                             0b10001,
//                             0b10001,
//                             0b10001,
//                             0b01110,
//                             0b00000,
//                           }
//                );
//
//  // Zeichen 2 (oder 10) soll ein grosses Ue (Umlaut) werden:
//  lcd1.defChar( 2,
//                (uint8_t[]){ 0b10001,
//                             0b00000,
//                             0b10001,
//                             0b10001,
//                             0b10001,
//                             0b10001,
//                             0b01110,
//                             0b00000,
//                           }
//                );
//
//  // Ausgeben: A, Umlaut Ae, a, Umlaut ae, O, Umlaut Oe, o, Umlaut oe,
//  // U, Umlaut Ue, u, Umlaut ue
//  // Anstelle des Zeichens 0 kann genausogut 8 ausgegeben werden,
//  // analog statt 1 die 9, statt 2 die 10 etc.. Durch die Verwendung
//  // von 8 statt 0 kann vermieden werden, dass das Zeichen 0 von
//  // printf() als Terminierung missverstanden wird.
//  //                      A Ae  a ae  O Oe  o oe  U Ue  u ue
//  lcd1.printf( 0, 3, 18, "A\x08 a\xe1 O\x01 o\xef U\x02 u\xf5" );
//
//  // Wenn keine Formatierung noetig ist wie bei printf(), geht auch
//  // printSZ(); das ist schneller:
//  lcd1.printSZ( " das waren die Umlaute" );
//
//  // printSZ() gibt es noch mit zwei zusaetzlichen Parametern vorweg
//  // für die Spalte und Zeile:
//  lcd1.printSZ( 36, 2, "..." );
//
//  // Entsprechend den beiden printSZ()-Varianten gibt es noch analog
//  // zwei printSZ_P(), bei denen der auszugebende String aus dem
//  // Programmspeicher geholt wird.
//}


// getestet mit LCD EA W204B-NLW 4x20
void demo_4x20()
{
  unsigned long long ull = 0x123456789abcdef0ull;

  // Initialisierung mit Angabe des Typs und Pinbelegung der
  // Anschluesse.
  AnyWare::LCD44780    lcd( AnyWare::LCD44780::LCD_Type_4x20,
                            PORTB, 0, // Port, Pin LCD-DB4
                            PORTB, 1, // Port, Pin LCD-DB5
                            PORTB, 2, // Port, Pin LCD-DB6
                            PORTB, 3, // Port, Pin LCD-DB7
                            PORTB, 4, // Port, Pin LCD-RS
                            PORTB, 5  // Port, Pin LCD-Enable1
                            );


  lcd.printf( 0, 0, 13, "Text-LCD 4x20" );
  //lcd.printf( 0, 1, 20, "%lx", ul );
  lcd.printf( 0, 2, 20, "%02x",
              (unsigned int)*(const unsigned char*)&ull
              );
}


//// getestet mit LCD 2x16 aus Pollin Addon-Board zu Atmel Evaluation Board
//void demo_2x16()
//{
//  // Initialisierung mit Angabe des Typs und Pinbelegung der
//  // Anschluesse.
//  AnyWare::LCD44780    lcd( AnyWare::LCD44780::LCD_Type_2x16,
//
//                            // Für Pollin Atmel Evaluation Board +
//                            // Pollin Addon Board:
//                            // Folgende Jumper setzen:
//                            //   PA0 - DB0 frei lassen
//                            //   PA1 - DB1 frei lassen
//                            //   PA2 - DB2 frei lassen
//                            //   PA3 - DB3 frei lassen
//                            //   PA4 - DB4 stecken
//                            //   PA5 - DB5 stecken
//                            //   PA6 - DB6 stecken
//                            //   PA7 - DB7 stecken
//                            //   PB1 - RW  frei lassen
//                            //   PB0 - E   stecken
//                            //   PB2 - RS  stecken
//                            // sowie eine Verbindung von LCD-R/W
//                            // nach GND (per Jumperkabel)
//                            PORTA, 4, // Port, Pin LCD-DB4
//                            PORTA, 5, // Port, Pin LCD-DB5
//                            PORTA, 6, // Port, Pin LCD-DB6
//                            PORTA, 7, // Port, Pin LCD-DB7
//                            PORTB, 2, // Port, Pin LCD-RS
//                            PORTB, 0  // Port, Pin LCD-Enable1
//
//                            //PORTD, 0, // Port, Pin LCD-DB4
//                            //PORTD, 1, // Port, Pin LCD-DB5
//                            //PORTD, 2, // Port, Pin LCD-DB6
//                            //PORTD, 3, // Port, Pin LCD-DB7
//                            //PORTD, 4, // Port, Pin LCD-RS
//                            //PORTD, 5  // Port, Pin LCD-Enable1
//                            );
//
//
//  lcd.printChar( 'H' );
//  lcd.printChar( 'a' );
//  lcd.printChar( 'l' );
//  lcd.printChar( 'l' );
//  lcd.printChar( 'o' );
//  lcd.printChar( '!' );
//  lcd.printChar( ' ' );
//
//  lcd.printf( 0, 1, 13, "Text-LCD 2x16" );
//
//  //lcd.setCursor( 0, 2 ); // erzeugt LCD_State_LineNumberInvalid
//  //lcd.printChar( 20, 0, 'x' );  // erzeugt LCD_State_ColumnNumberInvalid
//
//
//  switch( lcd.getState() )
//  {
//    case AnyWare::LCD44780::LCD_State_OK:
//      lcd.printf( 0, 0, 16, "OK" );
//      break;
//
//    case AnyWare::LCD44780::LCD_State_TypeInvalid:
//      lcd.printf( 0, 0, 16, "TypeInvalid" );
//      break;
//
//    case AnyWare::LCD44780::LCD_State_PinInvalid:
//      lcd.printf( 0, 0, 16, "PinInvalid" );
//      break;
//
//    case AnyWare::LCD44780::LCD_State_ColumnNumberInvalid:
//      lcd.printf( 0, 0, 16, "ColNumInvalid" );
//      break;
//
//    case AnyWare::LCD44780::LCD_State_LineNumberInvalid:
//      lcd.printf( 0, 0, 16, "LineNumInvalid" );
//      break;
//
//    case AnyWare::LCD44780::LCD_State_WidthTooSmall:
//      lcd.printf( 0, 0, 16, "WidthTooSmall" );
//      break;
//
//    default :
//      lcd.printf( 0, 0, 16, "?" );
//      break;
//  }
//}


int main( int nargs, char **args )
{
  //demo_4x40();
  demo_4x20();
  //demo_2x16();

  while( "warten aufs juengste Gericht" )
  {
  }

  return 0;
}

